/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		OSMemory.h

	Contains:	Prototypes for overridden new & delete, definition of OSMemory
				class which implements some memory leak debugging features.
					
	$Log: OSMemory.h,v $
	Revision 1.3  1999/12/21 01:43:34  lecroy
	*** empty log message ***
	
	Revision 1.2  1999/10/07 08:55:01  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.1.1.1  1999/09/22 23:01:19  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:38  rtucker
	initial checkin from rrv14
	
	

	Created: Tue, Mar 2, 1999 @ 4:23 PM
*/

#ifndef __OS_MEMORY_H__
#define __OS_MEMORY_H__

#include "OSHeaders.h"
#include "OSQueue.h"
#include "OSMutex.h"

class OSMemory
{
	public:
	
#if MEMORY_DEBUGGING
		//If memory debugging is on, clients can get access to data structures that give
		//memory status.
		static OSQueue*	GetTagQueue() { return &sTagQueue; }
		static OSMutex* GetTagQueueMutex() { return &sMutex;	}
		static UInt32	GetAllocatedMemory() { return sAllocatedBytes; }

		static void*	DebugNew(size_t size, UInt32 tag, Bool16 sizeCheck);
		static void		DebugDelete(void *mem);
		static Bool16		MemoryDebuggingTest();
		static void		ValidateMemoryQueue();

		struct TagElem
		{
			OSQueueElem elem;
			char tag[5];
			UInt32* tagPtr;
			UInt32 tagSize; //how big are objects of this type?
			UInt32 totMemory; //how much do they currently occupy
			UInt32 numObjects;//how many are there currently?
		};
#endif
		
		//When memory allocation fails, the server just exits. This sets the code
		//the server exits with
		static void SetMemoryError(SInt32 inErr);

		//Used by the new operator
		static void* New(size_t s, UInt32 tag, Bool16 sizeCheck);
		static void Delete(void* inMemory);
		
#if MEMORY_DEBUGGING
	private:
			
		struct MemoryDebugging
		{
			OSQueueElem elem;
			UInt32 tag;
			UInt32 size;
		};
		static OSQueue sMemoryQueue;
		static OSQueue sTagQueue;
		static UInt32  sAllocatedBytes;
		static OSMutex sMutex;
		
#endif
};

inline void* operator new(size_t, void* ptr) { return ptr;}

//The server assumes that memory allocation
//succeeds. If it doesn't succeed, we should exit the process immediately

//Memory allocation. We define a special new here simply for debugging purposes.
//This New routine allows the server to easily track memory leaks.
//Pass in a tag so that this memory allocation can later be identified
void* operator new(size_t s, UInt32 tag);
void* operator new[](size_t s, UInt32 tag);
void operator delete(void* mem);
void operator delete[](void* mem);
//make sure that the normal new operator is also overridden in case some code still uses it.
//these versions also do the debugging stuff if debugging is turned on
void* operator new (size_t s);
void* operator new[](size_t s);

#endif //__OS_MEMORY_H__
